"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.watch = void 0;
const generateShortId_1 = require("./generateShortId");
const Logger_1 = require("./Logger");
const subscribe_1 = require("./subscribe");
const fb_watchman_1 = require("fb-watchman");
const serialize_error_1 = require("serialize-error");
const log = Logger_1.Logger.child({
    namespace: 'watch',
});
const watch = (configurationInput) => {
    const { project, triggers, abortSignal } = {
        ...configurationInput,
    };
    const client = new fb_watchman_1.Client();
    return new Promise((resolve, reject) => {
        if (abortSignal) {
            abortSignal.addEventListener('abort', () => {
                client.end();
            }, {
                once: true,
            });
        }
        client.command(['watch-project', project], (error, response) => {
            var _a, _b, _c;
            if (error) {
                log.error({
                    error: (0, serialize_error_1.serializeError)(error),
                }, 'could not watch project');
                reject(error);
                client.end();
                return;
            }
            if ('warning' in response) {
                // eslint-disable-next-line no-console
                console.warn(response.warning);
            }
            log.info('watch established on %s relative_path %s', response.watch, response.relative_path);
            const subscriptions = [];
            for (const trigger of triggers) {
                subscriptions.push((0, subscribe_1.subscribe)(client, {
                    abortSignal,
                    debounce: trigger.debounce,
                    expression: trigger.expression,
                    id: (0, generateShortId_1.generateShortId)(),
                    interruptible: (_a = trigger.interruptible) !== null && _a !== void 0 ? _a : true,
                    name: trigger.name,
                    onChange: trigger.onChange,
                    onTeardown: trigger.onTeardown,
                    relativePath: response.relative_path,
                    retry: (_b = trigger.retry) !== null && _b !== void 0 ? _b : {
                        factor: 2,
                        maxTimeout: Number.POSITIVE_INFINITY,
                        minTimeout: 1000,
                        retries: 10,
                    },
                    throttleOutput: (_c = trigger.throttleOutput) !== null && _c !== void 0 ? _c : { delay: 1000 },
                    watch: response.watch,
                }));
            }
            // eslint-disable-next-line promise/prefer-await-to-then
            Promise.all(subscriptions).then(resolve).catch(reject);
        });
    });
};
exports.watch = watch;
